/*  graph library by array list
            12/Feb/2002    by Takeaki Uno
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */

#ifndef _sgraph_c_
#define _sgraph_c_

#include"sgraph.h"
#include"queue.c"


/*  initialization  */
void SGRAPH_init ( SGRAPH *G, int node_num, int edge_num, int arc_num ){
  int i;
  G->edge = G->in = G->out = 0;
  G->node_end = node_num;  // maximum number of vertices
  G->node_num = 0;         // current number of vertices
  G->edge_end = edge_num;  // maximum number of edges
  G->edge_num = 0;         // current number of edges
  G->arc_end = arc_num;    // maximum number of directed arcs
  G->arc_num = 0;          // current number of directed arcs
  G->edge = G->out = G->in = NULL;
  malloc2 ( G->buf, QUEUE_INT, (edge_num+arc_num+node_num*2) * 2, "SGRAPH_init: buf" );  // buffer memory for edge and arcs
  if ( edge_num > 0 ) malloc2 ( G->edge, QUEUE, node_num, "SGRAPH_init: edge" );     // QUEUE for each vertex storing the edges incident to it
  if ( arc_num > 0 ) malloc2 ( G->in, QUEUE, node_num, "SGRAPH_init: in" );
     // QUEUE for each vertex storing its in-arcs
  if ( arc_num > 0 ) malloc2 ( G->out, QUEUE, node_num, "SGRAPH_init: out" );
     // QUEUE for each vertex storing its out-arcs
  G->node1_num = 0;     // in bipartite case, {0,...,node1_num-1} is the vertex set 1, and the rest is the other
  G->node_w = NULL;  // vertex, edge, arc weight arrays are set to NULL
  G->edge_w = G->in_w = G->out_w = NULL;  
}

/* allocate memory for weight arrays */
void SGRAPH_malloc_weight ( SGRAPH *G ){
  malloc2 ( G->node_w, int, G->node_end, "SGRAPH_malloc_weight: node_w");
  malloc2 ( G->edge_w, QUEUE, G->node_end, "SGRAPH_init: edge_w" ); 
       // QUEUE for each vertex storing the edge weights
  malloc2 ( G->in_w, QUEUE, G->node_end, "SGRAPH_init: in_w" );
       // QUEUE for each vertex storing the in-arc weights
  malloc2 ( G->out_w, QUEUE, G->node_end, "SGRAPH_init: out_w" );  
       // QUEUE for each vertex storing the out-arc weights
}

/* copy graph G to graph G2. Underconstruction */
void SGRAPH_cpy ( SGRAPH *G2, SGRAPH *G ){}

/* free graph object  */
void SGRAPH_end ( SGRAPH *G ){
  int i;
  if ( G->buf == NULL ){
    for ( i=0 ; i<G->node_end ; i++ ){
      QUEUE_end ( &(G->edge[i]) );
      QUEUE_end ( &(G->in[i]) );
      QUEUE_end ( &(G->out[i]) );
      QUEUE_end ( &(G->edge_w[i]) );
      QUEUE_end ( &(G->in_w[i]) );
      QUEUE_end ( &(G->out_w[i]) );
    }
  } else free2 ( G->buf );
  
  free2 ( G->edge );
  free2 ( G->in );
  free2 ( G->out );
  free2 ( G->node_w );
  free2 ( G->edge_w );
  free2 ( G->in_w );
  free2 ( G->out_w );
}

/* return the position of v in the edge QUEUE of u, if v is adjacent to u
   if u is not adjacent to v, then return -1 */
int SGRAPH_inc ( SGRAPH *G, int u, int v ){
  int i, vv;
  QUEUE_FE_LOOP ( G->edge[u], i, vv )
    if ( vv == v ) return (i);
  return (-1);
}

/* return the position of v in the out/in-arc QUEUE of u, if v is adjacent to u
   if u is not adjacent to v, then return -1 */
int SGRAPH_out_inc ( SGRAPH *G, int u, int v ){
  int i;
  QUEUE_F_LOOP ( G->out[u], i )
    if ( G->out[u].q[i] == v ) return (i);
  return (-1);
}
int SGRAPH_in_inc ( SGRAPH *G, int u, int v ){
  int i;
  QUEUE_F_LOOP ( G->in[v], i )
    if ( G->in[v].q[i] == u ) return (i);
  return (-1);
}



/*  make an edge between u and v.
   If they are already connected, it will be a multiple edge */
void SGRAPH_edge_mk ( SGRAPH *G, int u, int v ){
  if ( u < 0 || v < 0 || v >= G->node_end || u >= G->node_end ){
    printf ("SGRAPH_edge_mk: vertex is out of range. u=%d v=%d\n", u, v);
    exit(1);
  }
  QUEUE_ins ( &(G->edge[u]), v );
  QUEUE_ins ( &(G->edge[v]), u );
  G->edge_num ++;
}

/*  make an arc between u and v.
   If they are already connected, it will be a multiple arc */
void SGRAPH_arc_mk ( SGRAPH *G, int u, int v ){
  if ( u < 0 || v < 0 || v >= G->node_end || u >= G->node_end ){
    printf ("SGRAPH_arc_mk: vertex is out of range. u=%d v=%d\n", u, v);
    exit(1);
  }
  QUEUE_ins ( &(G->out[u]), v );
  QUEUE_ins ( &(G->in[v]), u );
  G->arc_num ++;
}



/* Delete the edge connecting u and v. If edge (u,v) does not exist, nothing will occur. */
void SGRAPH_edge_rm ( SGRAPH *G, int u, int v ){
  int i;
  i = SGRAPH_inc ( G, u, v );
  if ( i >= 0 )
      QUEUE_rm ( &(G->edge[u]), i );
  i = SGRAPH_inc ( G, v, u );
  if ( i >= 0 )
      QUEUE_rm ( &(G->edge[v]), i );
  G->edge_num--;
}

/* Delete the arc connecting u and v. If arc (u,v) does not exist, nothing will occur. */
void SGRAPH_arc_rm ( SGRAPH *G, int u, int v ){
  int i;
  i = SGRAPH_out_inc ( G, u, v );
  if ( i >= 0 )
      QUEUE_rm ( &(G->out[u]), i );
  i = SGRAPH_in_inc ( G, v, u );
  if ( i >= 0 )
      QUEUE_rm ( &(G->in[v]), i );
  G->arc_num--;
}


/* return the maximum degree of the graph */
int SGRAPH_max_degree ( SGRAPH *G ){
  int i, m=0, k;
  for ( i=0 ; i<G->node_end ; i++ ){
    k = SGRAPH_DEG(*G,i);
    if ( k > m ) m = k;
  }
  return ( m );
}
int SGRAPH_max_in_degree ( SGRAPH *G ){
  int i, m=0, k;
  for ( i=0 ; i<G->node_end ; i++ ){
    k = SGRAPH_DEG(*G,i);
    if ( k > m ) m = k;
  }
  return ( m );
}
int SGRAPH_max_out_degree ( SGRAPH *G ){
  int i, m=0, k;
  for ( i=0 ; i<G->node_end ; i++ ){
    k = SGRAPH_OUT_DEG(*G,i);
    if ( k > m ) m = k;
  }
  return ( m );
}

/*  print graph by numbers  */
void SGRAPH_print ( SGRAPH *G ){
  int i, j, v, e;
  
  printf ("#node %d, #edge %d, #arc %d\n", G->node_end, G->edge_num, G->arc_num);
  for ( i=0 ; i<G->node_num ; i++ ){
    printf ("NODE %d ", i );
    if ( G->node_w ) printf ("(%d)", G->node_w[i] );
    printf (" >>\n");
    if ( G->edge ){
      if ( G->edge_w && G->edge_w[i].q[j] )
         printf ("(%d)", G->edge_w[i].q[j]);
      if ( QUEUE_LENGTH(G->edge[i]) ){
        printf ("    edge      : ");
        QUEUE_FE_LOOP ( G->edge[i], j, v ){
          printf ("%d", v );
          if ( G->edge_w && G->edge_w[i].q[j] )
              printf ("(%d)", G->edge_w[i].q[j]);
          printf (",");
        }
        printf ("\n");
      }
    }
    if ( G->in ){
      if ( QUEUE_LENGTH(G->in[i]) ){
        printf ("    in-arc      : ");
        QUEUE_F_LOOP ( G->in[i], j ){
          printf ("%d", G->in[i].q[j]);
          if ( G->in_w )
              printf ("(%d)", G->in_w[i].q[j]);
          printf (",");
        }
        printf ("\n");
      }
    }
    if ( G->out ){
      if ( QUEUE_LENGTH(G->out[i]) ){
        printf ("    out-arc      : ");
       QUEUE_FE_LOOP_ ( G->out[i], j, e ){
          printf ("%d", e);
          if ( G->out_w )
              printf ("(%d)", G->out_w[i].q[j]);
          printf (",");
        }
        printf ("\n");
      }
    }
  }
}

/* Output a graph to file
  Vertices, edges, arcs less than node_num, edge_num, arc_num are written to the file. Input parameters are
  (graph) (file name) (0 if not write edge weights) (0 if not write vertex weights)*/

/*
  format of file:(including notifications to make input file)
   
  the ith row corresponds to node i-1, and
    ID list of nodes adjacent to i, and having ID > i, for undirected graph
    ID list of nodes adjacent to i by out-going arc of i, for directed graph
   Separator is ",", but graph load routine accepts any letter for 
    separator but not a number.
   If the graph has both edges and arcs, write them in two lines separately,
    so a node then uses two lines, and #nodes = #lines/2.
  
    ==  Notifications to make input file ==
   Notice that if 0th line has node 2, and the 2nd line has 0, then there
    will be multiple edge (0,2) and (2,0).
   The read routine does not make error with multiple edges, it is allowed.

   The ID of nodes begin from 0. After reading graph, node_num is set to
    node_end.

   Input file example, without weights, E={(0,1),(0,2),(1,1),(1,3),(2,3)}
===========
   1,2
   1 3
   3
   
   [EOF]
=========
   Nodes are 0,1, and 2, both edges and arcs exist, with node/edge/arc weights)
   5000,1,30
   0,50,1,20,
   100,1,3
   2,20
   200
   
   [EOF]
=======
   where node weights are 5000, 100, 200, and edges and their weights are
    (0,1),30,   (1,1),3
    arcs and their weights are (0,0),50,   (0,1), 20,   (1,2), 20

    In the case of bipartite graph, write the adjacent-node lists only for 
     the node in node set one.
     
    
*/
void SGRAPH_save ( SGRAPH *G, char *filename, int nf, int wf ){
  int i, j, e, c, f;
  FILE *fp;

    /* t@CJ */
  fopen2w ( fp, filename, "SGRAPH_save");
  for ( i=0 ; i<G->node_num ; i++ ){
    f = 0;
    if ( nf ){ f=1; fprintf (fp, "%d", G->node_w[i]);}  /* _d݂ */
    if ( G->edge_num > 0){
      c = 0;
      QUEUE_FE_LOOP_ ( G->edge[i], j, e ){ /* _ i ɐڑ}̃[v*/
        if ( e >= i ){   /* }2񏑂܂Ȃ悤ɁAID傫_ւ̎}܂Ȃ */
          if (f) fprintf (fp, ",");
          fprintf (fp, "%d", e ); /* אڂ钸_̏ */
          f = 1;
          if (wf) fprintf (fp,",%d",G->edge_w[i].q[j]); /* }d݂̏ */
	}
      }
      fprintf ( fp, "\n");
    }
    if ( G->arc_num > 0){ /* A[N̏݁Be_ɂāAoĂ}̂ݏނƂɂAdh */
      QUEUE_FE_LOOP_ ( G->out[i], j, e ){ /* _ i ɐڑ}̃[v*/
        if (f) fprintf (fp, ",");
        fprintf (fp, "%d", e ); /* אڂ钸_̏ */
        f = 1;
        if (wf) fprintf (fp,",%d",G->out_w[i].q[j]); /* }d݂̏ */
      }
      fprintf ( fp, "\n");
    }
  }
  fclose ( fp );
}


/* Ott@CǂݍށBǂݍރOt̑傫ɍ킹
   Otmۂ郁̑傫߂B́A
   (Ot) (t@C) (}ǂނ) (L}ǂނ) (m[hd݂邩) (}d݂邩) (2ǂ)*/
/* }ɊւāFesɂ́Ae_ɑ΂A̒_Y̑傫ȗאڒ_̂݋Lq邱 */
/* 2OtLqƂ́A_W1̒_̍ŝ݋LqB
   _W2ɂẮAY0n܂Ăꍇi܂蒸_WP
   _ƓY_uꍇj́AIɕ␳ */
/* L2Ot̏ꍇ́A_W12ւ̎}݂̂ǂݍ܂ */
/* L2OtŁA_W21ւ̎}ꍇ́Aʂ̃OtƂ
   ǂݍ݁AƂ猟node1_num ݒ肷邱ƁB */

/* }d݁E_d݂́Afile_count_number ̂ƂŁA_Y
  őɏd݂܂߂Ă܂߁As=nodeƂĂB2Ot̓Y
  ␳͗LA݂Ȃs̒_d݂͑S -1 ɂȂ */
void SGRAPH_load (SGRAPH *G, char *filename, int undirected, int digraph, int nf, int wf, int bipartite){
  int i, x, k, *num, edge_num, max_node, min_node, node_num, base=0, node_end;
  int en=0, an=0;
  FILE *fp;
  QUEUE_INT *buf, *buf_w;  /* buffer for storing all edges/arcs */

      /* t@CJA{ǂݍ */
  edge_num = file_count_number ( filename, &node_num, &max_node, &min_node );
  if (bipartite && min_node<node_num ) base = node_num;
  if ( nf || wf ) node_end = node_num+base;
    else node_end = MAX(node_num, max_node+1+base);

  if ( undirected ){
    if ( digraph ) SGRAPH_init ( G, node_end/2, edge_num, 1 );
    else SGRAPH_init ( G, node_end, edge_num, 0 );
  } else SGRAPH_init ( G, node_end, 0, edge_num );
  G->node_num = G->node_end;
  if (bipartite) G->node1_num = node_num;

  fopen2r ( fp, filename, "SGRAPH_load");
  buf = G->buf;
  num = intarray_malloc_const (G->node_end, 0);

  if (wf){
    malloc2(G->buf_w, QUEUE_INT,(edge_num+G->node_end)*2,"SGRAPH_load: out_w");
    buf_w = G->buf_w;
    if (digraph){
      malloc2(G->out_w, QUEUE, G->node_end, "SGRAPH_load: out_w");
      malloc2(G->in_w, QUEUE, G->node_end, "SGRAPH_load: in_w");
    }
    if (undirected) malloc2(G->edge_w,QUEUE,G->node_end,"SGRAPH_load: edge_w");
  }
  if (nf) malloc2 (G->node_w, int, G->node_end, "SGRAPH_load: node_w");

  for ( i=0 ; i<node_end ; i++ ){
      /* }Ɋւ鏈 */
    if ( nf ) G->node_w[i] = file_read_int (fp); /* _d݂̓ǂݍ */
    if ( undirected ){
      QUEUE_init_ ( &(G->edge[i]), edge_num, buf );
      if ( wf ) QUEUE_init_ ( &(G->edge_w[i]), edge_num, buf_w );
      do {
        x = file_read_int (fp) + base;
        if ( (FILE_err&4)==0 ){
          QUEUE_ins_ ( &(G->edge[i]), x );
          en++;
          if ( x > i ) num[x]++;
          if ( wf ){
            x = file_read_int (fp);
            if ( FILE_err&4 ) error ("SGRAPH_load: edge weight is absent");
            QUEUE_ins_ ( &(G->edge_w[i]), x );
          }
        }
      } while ( (FILE_err&3) == 0 );
      G->edge[i].end = G->edge[i].t + 1 + num[i];
      buf += G->edge[i].end;
      if ( wf ){
        G->edge_w[i].end = G->edge_w[i].t + 1 + num[i];
        buf_w += G->edge_w[i].end;
      }
    }
    if ( digraph ){
        /* L}Ɋւ鏈 */
      QUEUE_init_ ( &(G->out[i]), edge_num, buf );
      if ( wf ) QUEUE_init_ ( &(G->out_w[i]), edge_num, buf_w );
      do {
        x = file_read_int (fp) + base;
        if ( (FILE_err&4)==0 ){
          QUEUE_ins_ ( &(G->out[i]), x );
          an++;
          num[x]++;
          if ( wf ){
            x = file_read_int (fp);
            if ( FILE_err&4 ) error ("SGRAPH_load: arc weight is absent");
            QUEUE_ins_ ( &(G->out_w[i]), x );
          }
        }
      } while ( (FILE_err&3) == 0 );
      G->out[i].end = G->out[i].t + 1;
      buf += G->out[i].end;
      if ( wf ){
        G->out_w[i].end = G->out_w[i].t + 1;
        buf_w += G->out_w[i].end;
      }
    }
  }

  if ( digraph ){
    for ( i=0 ; i<G->node_end ; i++ ){
      QUEUE_init_ ( &(G->in[i]), num[i], buf );
      buf += num[i];
    }
  }
  for ( i=0 ; i<node_num ; i++ ){
    if ( undirected ){
      QUEUE_FE_LOOP_ ( G->edge[i], k, x ){
        if ( i<x ){
          QUEUE_ins_ ( &(G->edge[x]), i );
          if ( wf ) QUEUE_ins_ ( &(G->edge_w[x]), G->edge_w[i].q[k] );
        }
      }
    }
    if ( digraph ){
      QUEUE_FE_LOOP_ ( G->out[i], k, x ){
        if ( i<x ){
          QUEUE_ins_ ( &(G->in[x]), i );
          if ( wf ) QUEUE_ins_ ( &(G->in_w[x]), G->out_w[i].q[k] );
        }
      }
    }
  }
  G->edge_num = en;
  G->arc_num = an;
  fclose ( fp );
  free ( num );
}


/* Yiperm[i]Œu */
void SGRAPH_replace_index ( SGRAPH *G, int *perm ){
  int i, v, j;
  for ( i=0 ; i<G->node_end ; i++ ){
    QUEUE_FE_LOOP_ ( G->edge[i], j, v ){
      G->edge[i].q[j] = perm[v];
    }
  }
  if ( G->edge ) perm_struct ( G->edge, sizeof(QUEUE), perm, G->node_end );
  if ( G->in ){
    perm_struct ( G->in, sizeof(QUEUE), perm, G->node_end );
    perm_struct ( G->out, sizeof(QUEUE), perm, G->node_end );
  }
  if ( G->node_w ){
    perm_struct ( G->node_w, sizeof(int), perm, G->node_end );
  }
}
  /*
void perm_struct ( void *a, int unit, int *perm, int siz ){
  int i;
  char *s, *ss = a;
  malloc2 ( s, char, unit*siz, "perm_struct: s" );
  memcpy ( s, ss, unit*siz );
  for ( i=0 ; i<siz ; i++ )
    memcpy ( ss + unit*perm[i], s + unit*i, unit );
  free ( s );
}
*/

/* ڑ钸_\[gB
    flag!=0Ȃ΍~ɂ悤ɁÂ  */
void SGRAPH_sort_incident_edges ( SGRAPH *G, int flag ){
  int i, j, v, *x, z;
  malloc2 ( x, int, G->node_end, "SGRAPH_sort_incident_edge_: x");
  for ( i=0 ; i<G->node_end ; i++ ){
    x[i] = G->edge[i].t;
    QUEUE_FE_LOOP_ ( G->edge[i], j, v )
        if ( v <= i ) QUEUE_ins_ ( &(G->edge[v]), i );
    QUEUE_RMALL ( G->edge[i]);
  }
  for ( i=0 ; i<G->node_end ; i++ ){
    z = x[i]-QUEUE_LENGTH (G->edge[i]);
    QUEUE_BE_LOOP_ ( G->edge[i], j, v )
        G->edge[i].q[j+z] = v;
    G->edge[i].t = G->edge[i].s;
  }
  for ( i=0 ; i<G->node_end ; i++ ){
    for ( j=G->edge[i].t ; j<x[i] ; j++ ){
      v = G->edge[i].q[j];
      if ( v > i ) QUEUE_ins_ ( &(G->edge[v]), i );
    }
    G->edge[i].t = x[i];
  }
  free (x);
}

/* _d݂Ń\[gBflag!=0Ȃ΍~  */
int *SGRAPH_sort_node_w ( SGRAPH *G, int flag ){
  int *perm, *q;
  perm = init_perm ( G->node_end );
  qsort_struct_perm_ ( &(G->node_w[G->node1_num]), G->node_end-G->node1_num,
                       sizeof(int), perm, flag );
  if ( G->node1_num > 0 ) 
      qsort_struct_perm_ ( G->node_w, G->node_end, sizeof(int), perm, flag );
  SGRAPH_replace_index ( G, perm );
  q = inverse_perm (perm, G->node_end );
  free ( perm );
  return ( q );
}

/* _(edge[i].tL[ɂ)Ń\[gBflag!=0Ȃ΍~ */
int *SGRAPH_sort_node_degree ( SGRAPH *G, int flag ){
  int *perm, *q;
  perm = init_perm ( G->node_end );
  radix_sort ( &(G->edge[G->node1_num].t), G->node_end-G->node1_num,
              sizeof(QUEUE), 0, G->node_end, &(perm[G->node1_num]), flag);
  if ( G->node1_num > 0 )
      radix_sort ( &(G->edge[0].t), G->node1_num,
                 sizeof(QUEUE), 0, G->node_end, perm, flag);
  /*  qsort_struct_perm_ ( &(G->edge[G->node1_num].t), G->node_end-G->node1_num,
                       sizeof(QUEUE), &(perm[G->node1_num]), flag);
  if ( G->node1_num > 0 )
      qsort_struct_perm_ ( &(G->edge[0].t), G->node1_num, sizeof(QUEUE),
                           perm, flag); */
  SGRAPH_replace_index ( G, perm );
  q = inverse_perm (perm, G->node_end );
  free ( perm );
  return ( q );
}

/* d}AZt[v 
   sort_incident_edges ĂƉ */
void SGRAPH_simple ( SGRAPH *G ){
  int i, j, u, jj;
  for ( i=0 ; i<G->node_end ; i++ ){
    if ( QUEUE_LENGTH ( G->edge[i] ) == 0 ) continue;
    jj = G->edge[i].s;
    for ( j=jj=G->edge[i].s+1 ; j<G->edge[i].t ; j++ ){
      if ( G->edge[i].q[j] != G->edge[i].q[j-1] && G->edge[i].q[j] != i ){
        G->edge[i].q[jj] = G->edge[i].q[j];
        jj++;
      }
    }
    G->edge[i].t = jj;
  }
}





/*
main (){
  int i;
  SGRAPH G;
  SGRAPH_init ( &G, 20, 400, 400 );
  SGRAPH_malloc_weight ( &G );
  G.node1_num = 8;
  SGRAPH_mk_rand_graph2 ( &G, 8, 7 );
  for ( i=0; i<G.edge_num ; i++ ) G.edge_w[i]=(SGRAPH_KEY)(-i); 
  for ( i=0; i<G.arc_num ; i++ ) G.arc_w[i]=(SGRAPH_KEY)(-i); 
  for ( i=0; i<G.node_num ; i++ ) G.node_w[i]=(SGRAPH_KEY)i; 
  SGRAPH_save ( &G, "tt2", 1,1 );
} 
*/

/*
main (){
  int i;
  SGRAPH G;
  SGRAPH_load ( &G, "tt2", 10 );
  G.node1_num = SGRAPH_find_node1_num ( &G );
  SGRAPH_print ( &G );
  printf ("node1_num %d\n", G.node1_num);
  SGRAPH_save ( &G, "tt3", 1, 1 );
}
*/

/*
main (){
  SGRAPH G;
  int i, j;
  
  SGRAPH_load ( &G, "G.GRP", 1, 0, 1, 0, 1);
  SGRAPH_print ( &G );
  SGRAPH_save ( &G, "tmp.grp", 1, 0);
  

}
*/


#endif
