/*  graph library by array list
            12/Feb/2002    by Takeaki Uno
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */

#ifndef _sgraph_h_
#define _sgraph_h_

#include <stdio.h>
#include "stdlib2.h"
#include"queue.h"


/*  Otp̍\  */
typedef struct {
  int node_end;     /* #max vertices */
  int edge_end;     /* #max edges */
  int arc_end;      /* #max arcs */
  int node_num;     /* #current vertices */
  int edge_num;     /* #current edges */
  int arc_num;      /* #current arcs */
  QUEUE_INT *buf;   /* pointer to the buffer memory, used by queues */
  QUEUE_INT *buf_w; /* pointer to the buffer for weights */
  QUEUE *edge;     /* list(array) of edges incident to each vertex */
  QUEUE *in;       /* list(array) of in-coming arcs incident to each vertex */
  QUEUE *out;      /* list(array) of out-going arcs incident to each vertex */
  int node1_num;   /* the size of vertex set 1, bipartite case. otherwise 0 */
  int *node_w;    /* pointer to the node weight array(int) */
  QUEUE *edge_w; /* pointer to the edge weight array(int) */
  QUEUE *in_w;  /* pointer to the in-coming arc weight array(int)  */
  QUEUE *out_w;  /* pointer to the out-going arc weight array(int)  */
} SGRAPH;


/* #degree, in-coming degree, and out-going degree of vertex v */
#define SGRAPH_DEG(G,v)       ((G).edge[v].s>(G).edge[v].t?(G).edge[v].t+(G).edge[v].end-(G).edge[v].s:(G).edge[v].t-(G).edge[v].s)
#define SGRAPH_IN_DEG(G,v)    ((G).in[v].s>(G).in[v].t?(G).in[v].t+(G).in[v].end-(G).in[v].s:(G).in[v].t-(G).in[v].s)
#define SGRAPH_OUT_DEG(G,v)   ((G).out[v].s>(G).out[v].t?(G).out[v].t+(G).out[v].end-(G).out[v].s:(G).out[v].t-(G).out[v].s)



/*************** initialization/termination ***********************/

/*  initialization, termination, allocate arrays for weights, copy and duplication */
void SGRAPH_init ( SGRAPH *G, int node_num, int edge_num, int arc_num );
void SGRAPH_malloc_weight ( SGRAPH *G );
void SGRAPH_cpy ( SGRAPH *G2, SGRAPH *G );
void SGRAPH_end ( SGRAPH *G );


/********************* functions **********************************/

/* if there is edge (u,v), return the position of v in the edge
  list of u. Return -1 if not exist */
int SGRAPH_inc ( SGRAPH *G, int u, int v );

/* if there is arc (u,v) (arc(v,u)), return the position of v in the out-going
 (in-coming) arc list of u. Return -1 if not exist */
int SGRAPH_out_inc ( SGRAPH *G, int u, int v );
int SGRAPH_in_inc ( SGRAPH *G, int u, int v );

/* return maximum degree/out-going degree/in-coming degree of graph G */
int SGRAPH_max_degree ( SGRAPH *G );
int SGRAPH_max_in_degree ( SGRAPH *G );
int SGRAPH_max_out_degree ( SGRAPH *G );


/**************** addition/deletion **********************************/

/*  make/take/remove edge e as connecting vertices u and v,
 and  edge (u,v). 
  do nothing if when make already existing edge, or delete non-existing edge.
  with range check of parameters */

void SGRAPH_edge_mk ( SGRAPH *G, int u, int v );
void SGRAPH_edge_rm ( SGRAPH *G, int u, int v );
void SGRAPH_arc_mk ( SGRAPH *G, int u, int v );
void SGRAPH_arc_rm ( SGRAPH *G, int u, int v );


/******************* print routines *************************************/

/*  print graph by numbers  */
void SGRAPH_print ( SGRAPH *G );

/* Write the graph to file. Edges, arcs, and nodes from 0 to node_num/edge_num/arc_num are written to file. Parameters are
  (graph) (file name) (not write edge weight => 0) (not write node weight => 0) */
void SGRAPH_save ( SGRAPH *G, char *fname, int wf, int nf );

/* graph load routine. Allocate memory as much as the size of input file.
   parameters are, 
   (graph) (file name) (read edges?) (read arcs?) (read node weights?) (read edge weight?) (bipartite?) */
/* In the row of each vertex, write only vertices larger than it connected by an edge */
void SGRAPH_load ( SGRAPH *G, char *filename, int undirected, int digraph, int nf, int wf, int bipartite );


/*
  format of file:(including notifications to make input file)
   
  the ith row corresponds to node i-1, and
    ID list of nodes adjacent to i, and having ID > i, for undirected graph
    ID list of nodes adjacent to i by out-going arc of i, for directed graph
   Separator is ",", but graph load routine accepts any letter for 
    separator but not a number.
   If the graph has both edges and arcs, write them in two lines separately,
    so a node then uses two lines, and #nodes = #lines/2.
  
    ==  Notifications to make input file ==
   Notice that if 0th line has node 2, and the 2nd line has 0, then there
    will be multiple edge (0,2) and (2,0).
   The read routine does not make error with multiple edges, it is allowed.

   The ID of nodes begin from 0. After reading graph, node_num is set to
    node_end.

   Input file example, without weights, E={(0,1),(0,2),(1,1),(1,3),(2,3)}
===========
   1,2
   1 3
   3
   
   [EOF]
=========
   Nodes are 0,1, and 2, both edges and arcs exist, with node/edge/arc weights)
   5000,1,30
   0,50,1,20,
   100,1,3
   2,20
   200
   
   [EOF]
=======
   where node weights are 5000, 100, 200, and edges and their weights are
    (0,1),30,   (1,1),3
    arcs and their weights are (0,0),50,   (0,1), 20,   (1,2), 20

    In the case of bipartite graph, write the adjacent-node lists only for 
     the node in node set one.
     
    
*/
  

/*************************************************************************/
   

#endif 








